#include "LKH.h"
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>

GainType SolveCGTSP(int * CGTour);
static double GetTimeUsage(int who);
static double Euclidean2DCost(int * CGTour);

/*
 * This file contains the main function of the GLKH program.
 */

int main(int argc, char *argv[])
{
    int *CGTour;
    GainType Cost;

    /* Read the specification of the problem */
    if (argc >= 2)
        ParameterFileName = argv[1];
    ReadParameters();
    ReadProblem();

    assert(CGTour = (int *) malloc((Subclusters + 1) * sizeof(int)));
    Cost = SolveCGTSP(CGTour);
    Dimension = ProblemType != ATSP ? Subclusters : Subclusters * 2;
    if (OutputTourFileName) {
        TraceLevel = 1;
        WriteTour(OutputTourFileName, CGTour, Cost);
    }
    if (TourFileName && (Optimum == MINUS_INFINITY || Cost < Optimum)) {
        TraceLevel = 1;
        WriteTour(TourFileName, CGTour, Cost);
    }
    printff("Value = " GainFormat " (%0.2f)", Cost,
            WeightType == EUC_2D ? Euclidean2DCost(CGTour) :
            (double) Cost / Scale);
    if (Optimum != MINUS_INFINITY && Optimum != 0)
        printff(", Error = %0.2f%%", 100.0 * (Cost - Optimum) / Optimum);
    printff(", Time = %0.1f sec.\n\n",
            GetTimeUsage(RUSAGE_SELF) + GetTimeUsage(RUSAGE_CHILDREN));
    return 0;
}

static double GetTimeUsage(int who)
{
    struct rusage ru;
    getrusage(who, &ru);
    return ru.ru_utime.tv_sec + ru.ru_utime.tv_usec / 1000000.0;
}

static double Euclidean2DCost(int * CGTour)
{
    double Cost = 0;
    int i;
    for (i = 0; i < Dimension; i++) {
        double dx = NodeSet[CGTour[i]].X - NodeSet[CGTour[i + 1]].X;
        double dy = NodeSet[CGTour[i]].Y - NodeSet[CGTour[i + 1]].Y;
        Cost += sqrt(dx * dx + dy * dy);
    }
    return Cost;
}
